import pyaudio
import wave
import os
import datetime
import numpy as np
import matplotlib.pyplot as plt
from python_speech_features import *

def plot_mfcc(mic_data_in, plot_y, mfcc_n):

	mic_data = np.frombuffer(mic_data_in, dtype=np.short)
	wav_data_plot = mic_data[0:-1:300]
	plot_x = np.linspace(0, 5, 735)
	a = len(wav_data_plot)
	plot_y = np.append(plot_y, wav_data_plot)
	plot_y = plot_y[a:]
	plt.ylim(-40000, 40000)
	plt.yticks([-40000, -20000, 0, 20000, 40000])
	plt.plot(plot_x, plot_y, color='green')
	plt.grid()
	fig_name = '../pic/wavepic.jpg'
	plt.savefig(fig_name, bbox_inches='tight')
	plt.close()

	mfccdata = mfcc(mic_data, nfilt=int(mfcc_n / 2), winlen=(len(mic_data) / 44100), samplerate=44100,
					numcep=int(mfcc_n / 2), nfft=len(mic_data))
	mfccdata_ = mfccdata[0][1:]
	mfccdata__ = np.append(mfccdata_, 0)
	mfcc_ = mfccdata__ - mfccdata[0]
	mfcc_data = np.append(mfccdata, mfcc_)
	# fftdata = fft(mic_data, n=(fft_n * 4))#频率取1/4，这里乘4
	# fft_data = abs(fftdata)
	# fft_data = fft_data[:fft_n]
	#print(fft_data)

	return mfcc_data, fig_name, plot_y

# def record_wav(record_pertime):
# 	CHUNK = int(44100 * record_pertime)
# 	FORMAT = pyaudio.paInt16
# 	CHANNELS = 1
# 	RATE = 44100
# 	p = pyaudio.PyAudio()
# 	stream = p.open(format=FORMAT,
# 					channels=CHANNELS,
# 					rate=RATE,
# 					input=True,
# 					frames_per_buffer=CHUNK)
# 	file_name = './recorddata'
# 	word_name = os.path.exists(file_name)
# 	if not word_name:
# 		os.makedirs(file_name)
# 	while True:
# 		frames = []
# 		for i in range(0, int(RATE / CHUNK * record_pertime)):
# 			data = stream.read(CHUNK)
# 			print(data)
# 			frames.append(data)
# 		stream.stop_stream()
# 		stream.close()
# 		p.terminate()
# 		now_time = datetime.datetime.now().strftime('%Y-%m-%d_%H_%M_%S_%f')
# 		wav_name = now_time + ".wav"
# 		WAVE_OUTPUT_FILENAME = './recorddata' + '/' + wav_name
# 		wf = wave.open(WAVE_OUTPUT_FILENAME, 'wb')
# 		wf.setnchannels(CHANNELS)
# 		wf.setsampwidth(p.get_sample_size(FORMAT))
# 		print(p.get_sample_size(FORMAT))
# 		wf.setframerate(RATE)
# 		wf.writeframes(b''.join(frames))
# 		wf.close()
# 		return WAVE_OUTPUT_FILENAME

def wav_mfcc(path, mfcc_n):
	f = wave.open(path, "rb")
	params = f.getparams()
	nchannels, sampwidth, framerate, nframes = params[:4]
	str_date = f.readframes(nframes)
	f.close()
	wave_date = np.frombuffer(str_date, dtype=np.short)
	mfccdata = mfcc(wave_date, nfilt=int(mfcc_n/2), winlen=(len(wave_date)/44100), samplerate=44100, numcep=int(mfcc_n / 2), nfft=len(wave_date))
	mfccdata_ = mfccdata[0][1:]
	mfccdata__ = np.append(mfccdata_, 0)
	mfcc_ = mfccdata__ - mfccdata[0]
	mfcc_data = np.append(mfccdata, mfcc_)
	return mfcc_data






import numpy as np
import detect_mfcc_plot
import scipy.stats as st
import os
import datetime
def train(norfile_list, falfile_list, envfile_list, nor_per_time, mfcc_n, now_chdir):
    norfilenum = len(norfile_list)
    falfilenum = len(falfile_list)
    envfilenum = len(envfile_list)

    nor_gauss_all = np.empty((norfilenum, (mfcc_n + 1), 2), dtype=float)
    fal_gauss_all = np.empty((falfilenum, (mfcc_n + 1), 2), dtype=float)
    env_gauss_all = np.empty((envfilenum, (mfcc_n + 1), 2), dtype=float)
    detect_time = np.empty((1, 1, 1), dtype=float)
    detect_time[0][0][0] = nor_per_time[0]

    for x in range(norfilenum):

        gauss_list_all_T = []
        nor_fft_feature_list = []
        filename = norfile_list[x]
        os.chdir(filename)
        wav_files = os.listdir()

        for wav_file in wav_files:
            fft_ = detect_mfcc_plot.wav_mfcc(wav_file, mfcc_n)
            nor_fft_feature_list.append(fft_)
        nor_fft_feature_array = np.asarray(nor_fft_feature_list)  # 将list转为array，方便以后处理

        for p in range(mfcc_n):
            temp = nor_fft_feature_array[:, p]
            mean = np.mean(temp)
            std = np.std(temp)
            nor_gauss_all[x][p][0] = mean
            nor_gauss_all[x][p][1] = std
            gauss_list = []
            for m in range(len(wav_files)):
                gauss = st.norm.pdf(temp[m], nor_gauss_all[x][p][0], nor_gauss_all[x][p][1])
                gauss_list.append(gauss)
            gauss_list_all_T.append(gauss_list)
        gauss_list_all_T = np.asarray(gauss_list_all_T)
        gauss_list_all = gauss_list_all_T.T
        #print(gauss_list_all.shape)
        mean_list = []
        for i in range(len(gauss_list_all)):
            mean = np.mean(gauss_list_all[i])
            mean_list.append(mean)
        nor_gauss_all[x][mfcc_n][0] = np.mean(mean_list)
        nor_gauss_all[x][mfcc_n][1] = np.std(mean_list)

    for x in range(falfilenum):
        gauss_list_all_T = []
        fal_fft_feature_list = []
        filename = falfile_list[x]
        os.chdir(filename)
        wav_files = os.listdir()

        for wav_file in wav_files:
            fft_ = detect_mfcc_plot.wav_mfcc(wav_file, mfcc_n)
            fal_fft_feature_list.append(fft_)
        fal_fft_feature_array = np.asarray(fal_fft_feature_list)  # 将list转为array，方便以后处理

        for p in range(mfcc_n):
            temp = fal_fft_feature_array[:, p]
            mean = np.mean(temp)
            std = np.std(temp)
            fal_gauss_all[x][p][0] = mean
            fal_gauss_all[x][p][1] = std
            gauss_list = []
            for m in range(len(wav_files)):
                gauss = st.norm.pdf(temp[m], fal_gauss_all[x][p][0], fal_gauss_all[x][p][1])
                gauss_list.append(gauss)
            gauss_list_all_T.append(gauss_list)
        gauss_list_all_T = np.asarray(gauss_list_all_T)
        gauss_list_all = gauss_list_all_T.T
        mean_list = []
        for i in range(len(gauss_list_all)):
            mean = np.mean(gauss_list_all[i])
            mean_list.append(mean)
        fal_gauss_all[x][mfcc_n][0] = np.mean(mean_list)
        fal_gauss_all[x][mfcc_n][1] = np.std(mean_list)

    for x in range(envfilenum):
        gauss_list_all_T = []
        env_fft_feature_list = []
        filename = envfile_list[x]
        os.chdir(filename)
        wav_files = os.listdir()

        for wav_file in wav_files:
            fft_ = detect_mfcc_plot.wav_mfcc(wav_file, mfcc_n)
            env_fft_feature_list.append(fft_)
        env_fft_feature_array = np.asarray(env_fft_feature_list)  # 将list转为array，方便以后处理

        for p in range(mfcc_n):
            temp = env_fft_feature_array[:, p]
            mean = np.mean(temp)
            std = np.std(temp)
            env_gauss_all[x][p][0] = mean
            env_gauss_all[x][p][1] = std
            gauss_list = []
            for m in range(len(wav_files)):
                gauss = st.norm.pdf(temp[m], env_gauss_all[x][p][0], env_gauss_all[x][p][1])
                gauss_list.append(gauss)
            gauss_list_all_T.append(gauss_list)
        gauss_list_all_T = np.asarray(gauss_list_all_T)
        gauss_list_all = gauss_list_all_T.T
        mean_list = []
        for i in range(len(gauss_list_all)):
            mean = np.mean(gauss_list_all[i])
            mean_list.append(mean)
        env_gauss_all[x][mfcc_n][0] = np.mean(mean_list)
        env_gauss_all[x][mfcc_n][1] = np.std(mean_list)

    os.chdir(now_chdir)
    file_name = './model'
    word_name = os.path.exists(file_name)
    if not word_name:
        os.makedirs(file_name)
    modeldir = now_chdir + '\model'
    os.chdir(modeldir)
    now_time = datetime.datetime.now().strftime('%Y-%m-%d_%H_%M_%S_%f')
    file_name = 'model' + now_time + '.npy'
    np.save(file=file_name, arr=(nor_gauss_all, fal_gauss_all, env_gauss_all, detect_time))



from setuptools import setup
setup(name='hhqtrain',
      version='0.0.1',
      description='clinical trial information retriver',
      url='http://github.com/',
      author='hhq',
      author_email='779613975@qq.com',
      license='MIT',
      packages=['hhqtrain'],
      zip_safe=False)